@@ -73,14 +73,14 @@ N_Vector_ID N_VGetVectorID_OpenMPDEV(N_Vector v)
  * Function to create a new empty vector
  */
 
-N_Vector N_VNewEmpty_OpenMPDEV(sunindextype length)
+N_Vector N_VNewEmpty_OpenMPDEV(sunindextype length, SUNContext sunctx)
 {
   N_Vector v;
   N_VectorContent_OpenMPDEV content;
 
   /* Create an empty vector object */
   v = NULL;
-  v = N_VNewEmpty();
+  v = N_VNewEmpty(sunctx);
   if (v == NULL) return(NULL);
 
   /* Attach operations */
@@ -131,6 +131,9 @@ N_Vector N_VNewEmpty_OpenMPDEV(sunindextype length)
   v->ops->nvwsqrsumlocal     = N_VWSqrSumLocal_OpenMPDEV;
   v->ops->nvwsqrsummasklocal = N_VWSqrSumMaskLocal_OpenMPDEV;
 
+  /* single buffer reduction operations */
+  v->ops->nvdotprodmultilocal = N_VDotProdMulti_OpenMPDEV;
+
   /* Create content */
   content = NULL;
   content = (N_VectorContent_OpenMPDEV) malloc(sizeof *content);
@@ -224,25 +227,7 @@ N_Vector N_VMake_OpenMPDEV(sunindextype length, realtype *h_vdata,
 
 N_Vector *N_VCloneVectorArray_OpenMPDEV(int count, N_Vector w)
 {
-  N_Vector *vs;
-  int j;
-
-  if (count <= 0) return(NULL);
-
-  vs = NULL;
-  vs = (N_Vector *) malloc(count * sizeof(N_Vector));
-  if(vs == NULL) return(NULL);
-
-  for (j = 0; j < count; j++) {
-    vs[j] = NULL;
-    vs[j] = N_VClone_OpenMPDEV(w);
-    if (vs[j] == NULL) {
-      N_VDestroyVectorArray_OpenMPDEV(vs, j-1);
-      return(NULL);
-    }
-  }
-
-  return(vs);
+  return(N_VCloneVectorArray(count, w));
 }
 
 /* ----------------------------------------------------------------------------
@@ -251,25 +236,7 @@ N_Vector *N_VCloneVectorArray_OpenMPDEV(int count, N_Vector w)
 
 N_Vector *N_VCloneVectorArrayEmpty_OpenMPDEV(int count, N_Vector w)
 {
-  N_Vector *vs;
-  int j;
-
-  if (count <= 0) return(NULL);
-
-  vs = NULL;
-  vs = (N_Vector *) malloc(count * sizeof(N_Vector));
-  if(vs == NULL) return(NULL);
-
-  for (j = 0; j < count; j++) {
-    vs[j] = NULL;
-    vs[j] = N_VCloneEmpty_OpenMPDEV(w);
-    if (vs[j] == NULL) {
-      N_VDestroyVectorArray_OpenMPDEV(vs, j-1);
-      return(NULL);
-    }
-  }
-
-  return(vs);
+  return(N_VCloneEmptyVectorArray(count, w));
 }
 
 /* ----------------------------------------------------------------------------
@@ -278,12 +245,7 @@ N_Vector *N_VCloneVectorArrayEmpty_OpenMPDEV(int count, N_Vector w)
 
 void N_VDestroyVectorArray_OpenMPDEV(N_Vector *vs, int count)
 {
-  int j;
-
-  for (j = 0; j < count; j++) N_VDestroy_OpenMPDEV(vs[j]);
-
-  free(vs); vs = NULL;
-
+  N_VDestroyVectorArray(vs, count);
   return;
 }
 
@@ -421,12 +383,12 @@ N_Vector N_VCloneEmpty_OpenMPDEV(N_Vector w)
 
   /* Create vector */
   v = NULL;
-  v = N_VNewEmpty();
+  v = N_VNewEmpty(w->sunctx);
   if (v == NULL) return(NULL);
 
   /* Attach operations */
   if (N_VCopyOps(w, v)) { N_VDestroy(v); return(NULL); }
-  
+
   /* Create content */
   content = NULL;
   content = (N_VectorContent_OpenMPDEV) malloc(sizeof *content);
@@ -2847,6 +2809,8 @@ int N_VEnableFusedOps_OpenMPDEV(N_Vector v, booleantype tf)
     v->ops->nvwrmsnormmaskvectorarray      = N_VWrmsNormMaskVectorArray_OpenMPDEV;
     v->ops->nvscaleaddmultivectorarray     = N_VScaleAddMultiVectorArray_OpenMPDEV;
     v->ops->nvlinearcombinationvectorarray = N_VLinearCombinationVectorArray_OpenMPDEV;
+    /* enable single buffer reduction operations */
+    v->ops->nvdotprodmultilocal = N_VDotProdMultiLocal_OpenMPDEV;
   } else {
     /* disable all fused vector operations */
     v->ops->nvlinearcombination = NULL;
@@ -2860,6 +2824,8 @@ int N_VEnableFusedOps_OpenMPDEV(N_Vector v, booleantype tf)
     v->ops->nvwrmsnormmaskvectorarray      = NULL;
     v->ops->nvscaleaddmultivectorarray     = NULL;
     v->ops->nvlinearcombinationvectorarray = NULL;
+    /* disable single buffer reduction operations */
+    v->ops->nvdotprodmultilocal = NULL;
   }
 
   /* return success */
@@ -2912,10 +2878,13 @@ int N_VEnableDotProdMulti_OpenMPDEV(N_Vector v, booleantype tf)
   if (v->ops == NULL) return(-1);
 
   /* enable/disable operation */
-  if (tf)
-    v->ops->nvdotprodmulti = N_VDotProdMulti_OpenMPDEV;
-  else
-    v->ops->nvdotprodmulti = NULL;
+  if (tf) {
+    v->ops->nvdotprodmulti      = N_VDotProdMulti_OpenMPDEV;
+    v->ops->nvdotprodmultilocal = N_VDotProdMulti_OpenMPDEV;
+  } else {
+    v->ops->nvdotprodmulti      = NULL;
+    v->ops->nvdotprodmultilocal = NULL;
+  }
 
   /* return success */
   return(0);
