@@ -6,8 +6,8 @@ TEST(mathMixMatFun, cosh) {
     return cosh(x1);
   };
   stan::test::expect_common_unary_vectorized(f);
-  stan::test::expect_unary_vectorized(f, -2.6, -2, -1.2, -0.2, 0.5, 1, 1.3,
-                                      1.5);
+  stan::test::expect_unary_vectorized<stan::test::PromoteToComplex::Yes>(
+      f, -2.6, -2, -1.2, -0.2, 0.5, 1, 1.3, 1.5);
   stan::test::expect_complex_common(f);
 }
 
