@@ -6,7 +6,8 @@ TEST(mathMixMatFun, sqrt) {
     using stan::math::sqrt;
     return sqrt(x1);
   };
-  stan::test::expect_common_nonzero_unary_vectorized(f);
+  stan::test::expect_common_nonzero_unary_vectorized<
+      stan::test::PromoteToComplex::No>(f);
   stan::test::expect_unary_vectorized(f, -6, -5.2, 1.3, 7, 10.7, 36, 1e6);
 
   // undefined with 0 in denominator
