@@ -434,7 +434,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestBeta) {
 
   var alpha = 9.0 / 5;
   var beta = 13.0 / 7;
-  AVEC theta = {alpha, beta};
+  std::vector<stan::math::var> theta = {alpha, beta};
   std::vector<double> x_r = {};
   std::vector<int> x_i = {};
   auto pdf = [](auto x, auto xc, std::ostream *msgs, auto theta, auto x_r,
@@ -444,8 +444,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestBeta) {
   var I = integrate_1d_impl(pdf, 0.0, 1.0, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(alpha, beta);
-  VEC g;
+  std::vector<stan::math::var> x{alpha, beta};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -458,7 +458,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestCauchy) {
 
   var mu = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {mu, sigma};
+  std::vector<stan::math::var> theta = {mu, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = -b;
   std::vector<double> x_r = {};
@@ -470,8 +470,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestCauchy) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{mu, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -483,7 +483,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestChiSquare) {
   using stan::math::var;
 
   var nu = 9.0 / 5;
-  AVEC theta = {nu};
+  std::vector<stan::math::var> theta = {nu};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -494,8 +494,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestChiSquare) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(nu);
-  VEC g;
+  std::vector<stan::math::var> x{nu};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
 }
@@ -507,7 +507,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestDoubleExponential) {
 
   var mu = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {mu, sigma};
+  std::vector<stan::math::var> theta = {mu, sigma};
   double a = -std::numeric_limits<double>::infinity();
   double b = mu.val();
   std::vector<double> x_r = {};
@@ -521,8 +521,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestDoubleExponential) {
           + integrate_1d_impl(pdf, b, -a, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{mu, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -534,7 +534,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestExponential) {
   using stan::math::var;
 
   var beta = 9.0 / 5;
-  AVEC theta = {beta};
+  std::vector<stan::math::var> theta = {beta};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -545,8 +545,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestExponential) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(beta);
-  VEC g;
+  std::vector<stan::math::var> x{beta};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
 }
@@ -558,7 +558,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestFrechet) {
 
   var alpha = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {alpha, sigma};
+  std::vector<stan::math::var> theta = {alpha, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -570,8 +570,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestFrechet) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(alpha, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{alpha, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -584,7 +584,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestGamma) {
 
   var alpha = 9.0 / 5;
   var beta = 13.0 / 7;
-  AVEC theta = {alpha, beta};
+  std::vector<stan::math::var> theta = {alpha, beta};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -596,8 +596,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestGamma) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(alpha, beta);
-  VEC g;
+  std::vector<stan::math::var> x{alpha, beta};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -610,7 +610,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestGumbel) {
 
   var mu = 9.0 / 5;
   var beta = 13.0 / 7;
-  AVEC theta = {mu, beta};
+  std::vector<stan::math::var> theta = {mu, beta};
   double b = std::numeric_limits<double>::infinity();
   double a = -b;
   std::vector<double> x_r = {};
@@ -622,8 +622,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestGumbel) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, beta);
-  VEC g;
+  std::vector<stan::math::var> x{mu, beta};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -635,7 +635,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestInvChiSquared) {
   using stan::math::var;
 
   var nu = 9.0 / 5;
-  AVEC theta = {nu};
+  std::vector<stan::math::var> theta = {nu};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -647,8 +647,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestInvChiSquared) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(nu);
-  VEC g;
+  std::vector<stan::math::var> x{nu};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
 }
@@ -660,7 +660,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestLogistic) {
 
   var mu = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {mu, sigma};
+  std::vector<stan::math::var> theta = {mu, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = -b;
   std::vector<double> x_r = {};
@@ -672,8 +672,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestLogistic) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{mu, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -686,7 +686,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestLogNormal) {
 
   var mu = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {mu, sigma};
+  std::vector<stan::math::var> theta = {mu, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -698,8 +698,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestLogNormal) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{mu, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -712,7 +712,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestNormal) {
 
   var mu = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {mu, sigma};
+  std::vector<stan::math::var> theta = {mu, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = -b;
   std::vector<double> x_r = {};
@@ -724,8 +724,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestNormal) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{mu, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -738,7 +738,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestPareto) {
 
   var m = 9.0 / 5;
   var alpha = 13.0 / 7;
-  AVEC theta = {m, alpha};
+  std::vector<stan::math::var> theta = {m, alpha};
   double b = std::numeric_limits<double>::infinity();
   var a = m;
   std::vector<double> x_r = {};
@@ -750,8 +750,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestPareto) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(m, alpha);
-  VEC g;
+  std::vector<stan::math::var> x{m, alpha};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -765,7 +765,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestPareto2) {
   var mu = 9.0 / 5;
   var lambda = 13.0 / 7;
   var alpha = 11.0 / 3;
-  AVEC theta = {mu, lambda, alpha};
+  std::vector<stan::math::var> theta = {mu, lambda, alpha};
   double b = std::numeric_limits<double>::infinity();
   var a = mu;
   std::vector<double> x_r = {};
@@ -777,8 +777,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestPareto2) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, lambda, alpha);
-  VEC g;
+  std::vector<stan::math::var> x{mu, lambda, alpha};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -791,7 +791,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestRayleigh) {
   using stan::math::var;
 
   var sigma = 13.0 / 7;
-  AVEC theta = {sigma};
+  std::vector<stan::math::var> theta = {sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -802,8 +802,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestRayleigh) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(sigma);
-  VEC g;
+  std::vector<stan::math::var> x{sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
 }
@@ -815,7 +815,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestScaledInvChiSquare) {
 
   var nu = 9.0 / 5;
   var s = 13.0 / 7;
-  AVEC theta = {nu, s};
+  std::vector<stan::math::var> theta = {nu, s};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -827,8 +827,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestScaledInvChiSquare) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(nu, s);
-  VEC g;
+  std::vector<stan::math::var> x{nu, s};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -842,7 +842,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestStudentT) {
   var nu = 11.0 / 3;
   var mu = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {nu, mu, sigma};
+  std::vector<stan::math::var> theta = {nu, mu, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = -b;
   std::vector<double> x_r = {};
@@ -854,8 +854,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestStudentT) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(nu, mu, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{nu, mu, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -871,7 +871,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestUniform) {
   var b = 13.0 / 7;
   std::vector<double> x_r = {};
   std::vector<int> x_i = {};
-  AVEC theta = {a, b};
+  std::vector<stan::math::var> theta = {a, b};
   auto pdf = [](auto x, auto xc, std::ostream *msgs, auto theta, auto x_r,
                 auto x_i) {
     return exp(stan::math::uniform_lpdf(x, theta[0], theta[1]));
@@ -879,8 +879,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestUniform) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(a, b);
-  VEC g;
+  std::vector<stan::math::var> x{a, b};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -893,7 +893,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestVonMises) {
 
   var mu = 9.0 / 5;
   var kappa = 13.0 / 7;
-  AVEC theta = {mu, kappa};
+  std::vector<stan::math::var> theta = {mu, kappa};
   double b = stan::math::pi() * 2;
   double a = 0;
   std::vector<double> x_r = {};
@@ -905,8 +905,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestVonMises) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(mu, kappa);
-  VEC g;
+  std::vector<stan::math::var> x{mu, kappa};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
@@ -919,7 +919,7 @@ TEST(StanMath_integrate_1d_impl_rev, TestWeibull) {
 
   var alpha = 9.0 / 5;
   var sigma = 13.0 / 7;
-  AVEC theta = {alpha, sigma};
+  std::vector<stan::math::var> theta = {alpha, sigma};
   double b = std::numeric_limits<double>::infinity();
   double a = 0;
   std::vector<double> x_r = {};
@@ -931,8 +931,8 @@ TEST(StanMath_integrate_1d_impl_rev, TestWeibull) {
   var I = integrate_1d_impl(pdf, a, b, 1e-8, msgs, theta, x_r, x_i);
   EXPECT_FLOAT_EQ(1, I.val());
 
-  AVEC x = createAVEC(alpha, sigma);
-  VEC g;
+  std::vector<stan::math::var> x{alpha, sigma};
+  std::vector<double> g;
   I.grad(x, g);
   EXPECT_FLOAT_EQ(1, 1 + g[0]);
   EXPECT_FLOAT_EQ(1, 1 + g[1]);
