@@ -10,24 +10,6 @@
 #include <type_traits>
 #include <string>
 
-/**
- * Tests for exact elementwise equality of input matrices of
- * the supplied type with he EXPECT_EQ macro from GTest.
- *
- * @param A first input matrix to compare
- * @param B second input matrix to compare
- * @param type types of elements in matrices A and B
- */
-#define EXPECT_TYPED_MATRIX_EQ(A, B, type)         \
-  {                                                \
-    const Eigen::Matrix<type, -1, -1>& A_eval = A; \
-    const Eigen::Matrix<type, -1, -1>& B_eval = B; \
-    EXPECT_EQ(A_eval.rows(), B_eval.rows());       \
-    EXPECT_EQ(A_eval.cols(), B_eval.cols());       \
-    for (int i = 0; i < A_eval.size(); i++)        \
-      EXPECT_EQ(A_eval(i), B_eval(i));             \
-  }
-
 /**
  * Tests for  exact elementwise equality of the input matrices
  * with the EXPECT_EQ macro from GTest.
@@ -102,35 +84,58 @@
  * @param B second input matrix to compare
  * @param DELTA the maximum allowed difference
  */
-#define EXPECT_MATRIX_NEAR(A, B, DELTA)         \
-  {                                             \
-    const Eigen::MatrixXd& A_eval = A;          \
-    const Eigen::MatrixXd& B_eval = B;          \
-    EXPECT_EQ(A_eval.rows(), B_eval.rows());    \
-    EXPECT_EQ(A_eval.cols(), B_eval.cols());    \
-    for (int i = 0; i < A_eval.size(); i++)     \
-      EXPECT_NEAR(A_eval(i), B_eval(i), DELTA); \
+#define EXPECT_MATRIX_NEAR(A, B, DELTA)                               \
+  {                                                                   \
+    using T_A = std::decay_t<decltype(A)>;                            \
+    using T_B = std::decay_t<decltype(B)>;                            \
+    const Eigen::Matrix<typename T_A::Scalar, T_A::RowsAtCompileTime, \
+                        T_A::ColsAtCompileTime>                       \
+        A_eval = A;                                                   \
+    const Eigen::Matrix<typename T_B::Scalar, T_B::RowsAtCompileTime, \
+                        T_B::ColsAtCompileTime>                       \
+        B_eval = B;                                                   \
+    EXPECT_EQ(A_eval.rows(), B_eval.rows());                          \
+    EXPECT_EQ(A_eval.cols(), B_eval.cols());                          \
+    for (int i = 0; i < A_eval.size(); i++)                           \
+      EXPECT_NEAR(A_eval(i), B_eval(i), DELTA);                       \
   }
 
 /**
- * Tests if any elementwise difference of the input matrices
- * of the specified type is greater than DELTA. This uses the
- * EXPECT_NEAR macro from GTest.
+ * Tests if given types are the same type.
  *
- * @param A first input matrix to compare
- * @param B second input matrix to compare
- * @param DELTA the maximum allowed difference
- * @param type of elements in the input matrices
+ * @param a first type
+ * @param b second type (code for this one can contain commas)
+ **/
+#define EXPECT_SAME_TYPE(a, ...)                                           \
+  EXPECT_TRUE((std::is_same<a, __VA_ARGS__>::value))                       \
+      << "Type a is" << stan::math::test::type_name<a>() << ". Type b is " \
+      << stan::math::test::type_name<__VA_ARGS__>();
+
+/**
+ * Tests if given value is of given type.
+ *
+ * @param type type
+ * @param value value (code for this one can contain commas)
+ **/
+#define EXPECT_TYPE(type, ...) EXPECT_SAME_TYPE(type, decltype(__VA_ARGS__))
+
+/**
+ * Count the number of times a substring is found in
+ * a supplied string.
+ *
+ * @param target substring to match in s
+ * @param s string to match count occurrences
+ * @return number of found occurrences of target in s
  */
-#define EXPECT_TYPED_MATRIX_NEAR(A, B, DELTA, type) \
-  {                                                 \
-    const Eigen::Matrix<type, -1, -1>& A_eval = A;  \
-    const Eigen::Matrix<type, -1, -1>& B_eval = B;  \
-    EXPECT_EQ(A_eval.rows(), B_eval.rows());        \
-    EXPECT_EQ(A_eval.cols(), B_eval.cols());        \
-    for (int i = 0; i < A_eval.size(); i++)         \
-      EXPECT_NEAR(A_eval(i), B_eval(i), DELTA);     \
-  }
+int count_matches(const std::string& target, const std::string& s) {
+  if (target.size() == 0)
+    return -1;  // error
+  int count = 0;
+  for (size_t pos = 0; (pos = s.find(target, pos)) != std::string::npos;
+       pos += target.size())
+    ++count;
+  return count;
+}
 
 /**
  * Tests if the expression throws the expected
@@ -164,67 +169,8 @@
 #define EXPECT_THROW_MSG(expr, T_e, msg) \
   EXPECT_THROW_MSG_WITH_COUNT(expr, T_e, msg, 1)
 
-/**
- * Tests if given types are the same type.
- *
- * @param a first type
- * @param b second type (code for this one can contain commas)
- **/
-#define EXPECT_SAME_TYPE(a, ...)                                           \
-  EXPECT_TRUE((std::is_same<a, __VA_ARGS__>::value))                       \
-      << "Type a is" << stan::math::test::type_name<a>() << ". Type b is " \
-      << stan::math::test::type_name<__VA_ARGS__>();
-
-/**
- * Count the number of times a substring is found in
- * a supplied string.
- *
- * @param target substring to match in s
- * @param s string to match count occurrences
- * @return number of found occurrences of target in s
- */
-int count_matches(const std::string& target, const std::string& s) {
-  if (target.size() == 0)
-    return -1;  // error
-  int count = 0;
-  for (size_t pos = 0; (pos = s.find(target, pos)) != std::string::npos;
-       pos += target.size())
-    ++count;
-  return count;
-}
-
-/**
- * Test for equality of the supplied types
- *
- * @tparam T1 first tpye
- * @tparam T2 second type
- */
-namespace test {
-template <typename T1, typename T2>
-void expect_same_type() {
-  bool b = std::is_same<T1, T2>::value;
-  EXPECT_TRUE(b);
-}
-}  // namespace test
-
 namespace stan {
 namespace test {
-template <int R, int C>
-void expect_type_vector(const Eigen::Matrix<double, R, C>& x) {
-  EXPECT_EQ(Eigen::Dynamic, R);
-  EXPECT_EQ(1, C);
-}
-template <int R, int C>
-void expect_type_row_vector(const Eigen::Matrix<double, R, C>& x) {
-  EXPECT_EQ(Eigen::Dynamic, C);
-  EXPECT_EQ(1, R);
-}
-
-template <int R, int C>
-void expect_type_matrix(const Eigen::Matrix<double, R, C>& x) {
-  EXPECT_EQ(Eigen::Dynamic, C);
-  EXPECT_EQ(Eigen::Dynamic, R);
-}
 
 auto make_sparse_matrix_random(int rows, int cols) {
   using eigen_triplet = Eigen::Triplet<double>;
