@@ -1,7 +1,9 @@
 #ifndef STAN_MODEL_INDEXING_ASSIGN_HPP
 #define STAN_MODEL_INDEXING_ASSIGN_HPP
 
-#include <stan/math/prim.hpp>
+#include <stan/math/prim/meta.hpp>
+#include <stan/math/prim/err.hpp>
+#include <stan/math/prim/fun/to_ref.hpp>
 #include <stan/model/indexing/access_helpers.hpp>
 #include <stan/model/indexing/index.hpp>
 #include <stan/model/indexing/rvalue_at.hpp>
@@ -52,7 +54,7 @@ template <
     typename T, typename U,
     require_t<std::is_assignable<std::decay_t<T>&, std::decay_t<U>>>* = nullptr>
 inline void assign(T&& x, U&& y, const char* name) {
-  x = std::forward<U>(y);
+  internal::assign_impl(x, std::forward<U>(y), name);
 }
 
 /**
@@ -95,8 +97,8 @@ template <typename Vec1, typename Vec2,
 inline void assign(Vec1&& x, const Vec2& y, const char* name,
                    const index_multi& idx) {
   const auto& y_ref = stan::math::to_ref(y);
-  stan::math::check_size_match("vector[multi] assign", "left hand side",
-                               idx.ns_.size(), name, y_ref.size());
+  stan::math::check_size_match("vector[multi] assign", name, idx.ns_.size(),
+                               "right hand side", y_ref.size());
   const auto x_size = x.size();
   for (int n = 0; n < y_ref.size(); ++n) {
     stan::math::check_range("vector[multi] assign", name, x_size, idx.ns_[n]);
@@ -124,22 +126,19 @@ template <typename Vec1, typename Vec2,
           require_all_not_std_vector_t<Vec1, Vec2>* = nullptr>
 inline void assign(Vec1&& x, const Vec2& y, const char* name,
                    index_min_max idx) {
-  stan::math::check_range("vector[min_max] min assign", name, x.size(),
-                          idx.min_);
-  stan::math::check_range("vector[min_max] max assign", name, x.size(),
-                          idx.max_);
-  if (idx.is_ascending()) {
+  if (likely(idx.max_ >= idx.min_)) {
+    stan::math::check_range("vector[min_max] min assign", name, x.size(),
+                            idx.min_);
+    stan::math::check_range("vector[min_max] max assign", name, x.size(),
+                            idx.max_);
     const auto slice_start = idx.min_ - 1;
     const auto slice_size = idx.max_ - slice_start;
-    stan::math::check_size_match("vector[min_max] assign", "left hand side",
-                                 slice_size, name, y.size());
-    x.segment(slice_start, slice_size) = y;
+    stan::math::check_size_match("vector[min_max] assign", name, slice_size,
+                                 "right hand side", y.size());
+    internal::assign_impl(x.segment(slice_start, slice_size), y, name);
   } else {
-    const auto slice_start = idx.max_ - 1;
-    const auto slice_size = idx.min_ - slice_start;
-    stan::math::check_size_match("vector[reverse_min_max] assign",
-                                 "left hand side", slice_size, name, y.size());
-    x.segment(slice_start, slice_size) = y.reverse();
+    stan::math::check_size_match("vector[negative_min_max] assign", name, 0,
+                                 "right hand side", y.size());
   }
 }
 
@@ -163,9 +162,10 @@ template <typename Vec1, typename Vec2,
           require_all_not_std_vector_t<Vec1, Vec2>* = nullptr>
 inline void assign(Vec1&& x, const Vec2& y, const char* name, index_min idx) {
   stan::math::check_range("vector[min] assign", name, x.size(), idx.min_);
-  stan::math::check_size_match("vector[min] assign", "left hand side",
-                               x.size() - idx.min_ + 1, name, y.size());
-  x.tail(x.size() - idx.min_ + 1) = y;
+  stan::math::check_size_match("vector[min] assign", name,
+                               x.size() - idx.min_ + 1, "right hand side",
+                               y.size());
+  internal::assign_impl(x.tail(x.size() - idx.min_ + 1), y, name);
 }
 
 /**
@@ -187,10 +187,15 @@ template <typename Vec1, typename Vec2,
           require_all_vector_t<Vec1, Vec2>* = nullptr,
           require_all_not_std_vector_t<Vec1, Vec2>* = nullptr>
 inline void assign(Vec1&& x, const Vec2& y, const char* name, index_max idx) {
-  stan::math::check_range("vector[max] assign", name, x.size(), idx.max_);
-  stan::math::check_size_match("vector[max] assign", "left hand side", idx.max_,
-                               name, y.size());
-  x.head(idx.max_) = y;
+  if (likely(idx.max_ > 0)) {
+    stan::math::check_range("vector[max] assign", name, x.size(), idx.max_);
+    stan::math::check_size_match("vector[max] assign", name, idx.max_,
+                                 "right hand side", y.size());
+    internal::assign_impl(x.head(idx.max_), y, name);
+  } else {
+    stan::math::check_size_match("vector[max < 1] assign", name, 0,
+                                 "right hand side", y.size());
+  }
 }
 
 /**
@@ -210,9 +215,9 @@ template <typename Vec1, typename Vec2,
           require_all_vector_t<Vec1, Vec2>* = nullptr,
           require_all_not_std_vector_t<Vec1, Vec2>* = nullptr>
 inline void assign(Vec1&& x, Vec2&& y, const char* name, index_omni /* idx */) {
-  stan::math::check_size_match("vector[omni] assign", "left hand side",
-                               x.size(), name, y.size());
-  x = std::forward<Vec2>(y);
+  stan::math::check_size_match("vector[omni] assign", name, x.size(),
+                               "right hand side", y.size());
+  internal::assign_impl(x, std::forward<Vec2>(y), name);
 }
 
 /**
@@ -235,10 +240,10 @@ template <typename Mat, typename RowVec,
           require_dense_dynamic_t<Mat>* = nullptr,
           require_row_vector_t<RowVec>* = nullptr>
 inline void assign(Mat&& x, const RowVec& y, const char* name, index_uni idx) {
-  stan::math::check_size_match("matrix[uni] assign", "left hand side columns",
-                               x.cols(), name, y.size());
   stan::math::check_range("matrix[uni] assign row", name, x.rows(), idx.n_);
-  x.row(idx.n_ - 1) = y;
+  stan::math::check_size_match("matrix[uni] assign columns", name, x.cols(),
+                               "right hand side size", y.size());
+  internal::assign_impl(x.row(idx.n_ - 1), y, name);
 }
 
 /**
@@ -261,10 +266,11 @@ template <typename Mat1, typename Mat2,
 inline void assign(Mat1&& x, const Mat2& y, const char* name,
                    const index_multi& idx) {
   const auto& y_ref = stan::math::to_ref(y);
-  stan::math::check_size_match("matrix[multi] assign", "left hand side rows",
-                               idx.ns_.size(), name, y.rows());
-  stan::math::check_size_match("matrix[multi] assign", "left hand side columns",
-                               x.cols(), name, y.cols());
+  stan::math::check_size_match("matrix[multi] assign rows", name,
+                               idx.ns_.size(), "right hand side rows",
+                               y.rows());
+  stan::math::check_size_match("matrix[multi] assign columns", name, x.cols(),
+                               "right hand side columns", y.cols());
   for (int i = 0; i < idx.ns_.size(); ++i) {
     const int n = idx.ns_[i];
     stan::math::check_range("matrix[multi] assign row", name, x.rows(), n);
@@ -289,11 +295,11 @@ inline void assign(Mat1&& x, const Mat2& y, const char* name,
 template <typename Mat1, typename Mat2,
           require_all_dense_dynamic_t<Mat1, Mat2>* = nullptr>
 inline void assign(Mat1&& x, Mat2&& y, const char* name, index_omni /* idx */) {
-  stan::math::check_size_match("matrix[omni] assign", "left hand side rows",
-                               x.rows(), name, y.rows());
-  stan::math::check_size_match("matrix[omni] assign", "left hand side columns",
-                               x.cols(), name, y.cols());
-  x = std::forward<Mat2>(y);
+  stan::math::check_size_match("matrix[omni] assign rows", name, x.rows(),
+                               "right hand side rows", y.rows());
+  stan::math::check_size_match("matrix[omni] assign columns", name, x.cols(),
+                               "right hand side columns", y.cols());
+  internal::assign_impl(x, std::forward<Mat2>(y), name);
 }
 
 /**
@@ -317,11 +323,11 @@ template <typename Mat1, typename Mat2,
 inline void assign(Mat1&& x, const Mat2& y, const char* name, index_min idx) {
   const auto row_size = x.rows() - (idx.min_ - 1);
   stan::math::check_range("matrix[min] assign row", name, x.rows(), idx.min_);
-  stan::math::check_size_match("matrix[min] assign", "left hand side rows",
-                               row_size, name, y.rows());
-  stan::math::check_size_match("matrix[min] assign", "left hand side columns",
-                               x.cols(), name, y.cols());
-  x.bottomRows(row_size) = y;
+  stan::math::check_size_match("matrix[min] assign rows", name, row_size,
+                               "right hand side rows", y.rows());
+  stan::math::check_size_match("matrix[min] assign columns", name, x.cols(),
+                               "right hand side columns", y.cols());
+  internal::assign_impl(x.bottomRows(row_size), y, name);
 }
 
 /**
@@ -343,12 +349,17 @@ template <typename Mat1, typename Mat2,
           require_dense_dynamic_t<Mat1>* = nullptr,
           require_matrix_t<Mat2>* = nullptr>
 inline void assign(Mat1&& x, const Mat2& y, const char* name, index_max idx) {
-  stan::math::check_range("matrix[max] assign row", name, x.rows(), idx.max_);
-  stan::math::check_size_match("matrix[max] assign", "left hand side rows",
-                               idx.max_, name, y.rows());
-  stan::math::check_size_match("matrix[max] assign", "left hand side columns",
-                               x.cols(), name, y.cols());
-  x.topRows(idx.max_) = y;
+  stan::math::check_size_match("matrix[max] assign columns", name, x.cols(),
+                               "right hand side columns", y.cols());
+  if (likely(idx.max_ > 0)) {
+    stan::math::check_range("matrix[max] assign row", name, x.rows(), idx.max_);
+    stan::math::check_size_match("matrix[max] assign rows", name, idx.max_,
+                                 "right hand side rows", y.rows());
+    internal::assign_impl(x.topRows(idx.max_), y, name);
+  } else {
+    stan::math::check_size_match("matrix[max < 1] assign rows", name, 0,
+                                 "right hand side rows", y.rows());
+  }
 }
 
 /**
@@ -370,27 +381,20 @@ template <typename Mat1, typename Mat2,
           require_dense_dynamic_t<Mat1>* = nullptr,
           require_matrix_t<Mat2>* = nullptr>
 inline void assign(Mat1&& x, Mat2&& y, const char* name, index_min_max idx) {
-  stan::math::check_range("matrix[min_max] max row indexing", name, x.rows(),
-                          idx.max_);
-  stan::math::check_range("matrix[min_max] min row indexing", name, x.rows(),
-                          idx.min_);
-  stan::math::check_size_match("matrix[min_max] assign",
-                               "left hand side columns", x.cols(), name,
-                               y.cols());
-  if (idx.is_ascending()) {
+  stan::math::check_size_match("matrix[min_max] assign columns", name, x.cols(),
+                               "right hand side columns", y.cols());
+  if (likely(idx.max_ >= idx.min_)) {
+    stan::math::check_range("matrix[min_max] min row indexing", name, x.rows(),
+                            idx.min_);
+    stan::math::check_range("matrix[min_max] max row indexing", name, x.rows(),
+                            idx.max_);
     const auto row_size = idx.max_ - idx.min_ + 1;
-    stan::math::check_size_match("matrix[min_max] assign",
-                                 "left hand side rows", row_size, name,
-                                 y.rows());
-    x.middleRows(idx.min_ - 1, row_size) = y;
-    return;
+    stan::math::check_size_match("matrix[min_max] assign rows", name, row_size,
+                                 "right hand side rows", y.rows());
+    internal::assign_impl(x.middleRows(idx.min_ - 1, row_size), y, name);
   } else {
-    const auto row_size = idx.min_ - idx.max_ + 1;
-    stan::math::check_size_match("matrix[reverse_min_max] assign",
-                                 "left hand side rows", row_size, name,
-                                 y.rows());
-    x.middleRows(idx.max_ - 1, row_size) = internal::colwise_reverse(y);
-    return;
+    stan::math::check_size_match("matrix[negative_min_max] assign rows", name,
+                                 0, "right hand side", y.rows());
   }
 }
 
@@ -414,64 +418,56 @@ template <typename Mat1, typename Mat2,
           require_dense_dynamic_t<Mat1>* = nullptr>
 inline void assign(Mat1&& x, Mat2&& y, const char* name, index_min_max row_idx,
                    index_min_max col_idx) {
-  stan::math::check_range("matrix[min_max, min_max] assign max row", name,
-                          x.rows(), row_idx.max_);
-  stan::math::check_range("matrix[min_max, min_max] assign min row", name,
-                          x.rows(), row_idx.min_);
-  stan::math::check_range("matrix[min_max, min_max] assign max column", name,
-                          x.cols(), col_idx.max_);
-  stan::math::check_range("matrix[min_max, min_max] assign min column", name,
-                          x.cols(), col_idx.min_);
-  if (row_idx.is_ascending()) {
-    if (col_idx.is_ascending()) {
-      auto row_size = row_idx.max_ - (row_idx.min_ - 1);
-      auto col_size = col_idx.max_ - (col_idx.min_ - 1);
-      stan::math::check_size_match("matrix[min_max, min_max] assign",
-                                   "left hand side rows", row_size, name,
-                                   y.rows());
-      stan::math::check_size_match("matrix[min_max, min_max] assign",
-                                   "left hand side columns", col_size, name,
-                                   y.cols());
-      x.block(row_idx.min_ - 1, col_idx.min_ - 1, row_size, col_size) = y;
-      return;
-    } else {
-      auto row_size = row_idx.max_ - (row_idx.min_ - 1);
-      auto col_size = col_idx.min_ - (col_idx.max_ - 1);
-      stan::math::check_size_match("matrix[min_max, reverse_min_max] assign",
-                                   "left hand side rows", row_size, name,
-                                   y.rows());
-      stan::math::check_size_match("matrix[min_max, reverse_min_max] assign",
-                                   "left hand side columns", col_size, name,
-                                   y.cols());
-      x.block(row_idx.min_ - 1, col_idx.max_ - 1, row_size, col_size)
-          = internal::rowwise_reverse(y);
-      return;
-    }
+  if (likely((row_idx.max_ >= row_idx.min_)
+             && (col_idx.max_ >= col_idx.min_))) {
+    stan::math::check_range("matrix[min_max, min_max] assign min row", name,
+                            x.rows(), row_idx.min_);
+
+    stan::math::check_range("matrix[min_max, min_max] assign min column", name,
+                            x.cols(), col_idx.min_);
+    stan::math::check_range("matrix[min_max, min_max] assign max row", name,
+                            x.rows(), row_idx.max_);
+    stan::math::check_range("matrix[min_max, min_max] assign max column", name,
+                            x.cols(), col_idx.max_);
+    auto row_size = row_idx.max_ - (row_idx.min_ - 1);
+    auto col_size = col_idx.max_ - (col_idx.min_ - 1);
+    stan::math::check_size_match("matrix[min_max, min_max] assign rows", name,
+                                 row_size, "right hand side rows", y.rows());
+    stan::math::check_size_match("matrix[min_max, min_max] assign columns",
+                                 name, col_size, "right hand side columns",
+                                 y.cols());
+    internal::assign_impl(
+        x.block(row_idx.min_ - 1, col_idx.min_ - 1, row_size, col_size), y,
+        name);
   } else {
-    if (col_idx.is_ascending()) {
-      auto row_size = row_idx.min_ - (row_idx.max_ - 1);
-      auto col_size = col_idx.max_ - (col_idx.min_ - 1);
-      stan::math::check_size_match("matrix[reverse_min_max, min_max] assign",
-                                   "left hand side rows", row_size, name,
-                                   y.rows());
-      stan::math::check_size_match("matrix[reverse_min_max, min_max] assign",
-                                   "left hand side columns", col_size, name,
+    // Check the indexing is valid but don't actually do assignment
+    if (row_idx.max_ >= row_idx.min_) {
+      stan::math::check_range(
+          "matrix[min_max, negative_min_max] assign min row", name, x.rows(),
+          row_idx.min_);
+      stan::math::check_range(
+          "matrix[min_max, negative_min_max] assign max row", name, x.rows(),
+          row_idx.max_);
+      stan::math::check_size_match("matrix[min_max, negative_min_max] assign",
+                                   name, 0, "right hand side columns",
                                    y.cols());
-      x.block(row_idx.max_ - 1, col_idx.min_ - 1, row_size, col_size)
-          = internal::colwise_reverse(y);
-      return;
+
+    } else if (col_idx.max_ >= col_idx.min_) {
+      stan::math::check_range(
+          "matrix[negative_min_max, min_max] assign min column", name, x.cols(),
+          col_idx.min_);
+      stan::math::check_range(
+          "matrix[negative_min_max, min_max] assign max column", name, x.cols(),
+          col_idx.max_);
+      stan::math::check_size_match("matrix[min_max, min_max] assign rows", name,
+                                   0, "right hand side", y.rows());
     } else {
-      auto row_size = row_idx.min_ - (row_idx.max_ - 1);
-      auto col_size = col_idx.min_ - (col_idx.max_ - 1);
       stan::math::check_size_match(
-          "matrix[reverse_min_max, reverse_min_max] assign",
-          "left hand side rows", row_size, name, y.rows());
+          "matrix[negative_min_max, negative_min_max] assign rows", name, 0,
+          "right hand side", y.rows());
       stan::math::check_size_match(
-          "matrix[reverse_min_max, reverse_min_max] assign",
-          "left hand side columns", col_size, name, y.cols());
-      x.block(row_idx.max_ - 1, col_idx.max_ - 1, row_size, col_size)
-          = y.reverse();
-      return;
+          "matrix[negative_min_max, negative_min_max] assign cols", name, 0,
+          "right hand side", y.cols());
     }
   }
 }
@@ -526,8 +522,9 @@ inline void assign(Mat1&& x, const Vec& y, const char* name, index_uni row_idx,
   const auto& y_ref = stan::math::to_ref(y);
   stan::math::check_range("matrix[uni, multi] assign row", name, x.rows(),
                           row_idx.n_);
-  stan::math::check_size_match("matrix[uni, multi] assign", "left hand side",
-                               col_idx.ns_.size(), name, y_ref.size());
+  stan::math::check_size_match("matrix[uni, multi] assign", name,
+                               col_idx.ns_.size(), "right hand side",
+                               y_ref.size());
   for (int i = 0; i < col_idx.ns_.size(); ++i) {
     stan::math::check_range("matrix[uni, multi] assign column", name, x.cols(),
                             col_idx.ns_[i]);
@@ -557,11 +554,11 @@ template <typename Mat1, typename Mat2,
 inline void assign(Mat1&& x, const Mat2& y, const char* name,
                    const index_multi& row_idx, const index_multi& col_idx) {
   const auto& y_ref = stan::math::to_ref(y);
-  stan::math::check_size_match("matrix[multi,multi] assign row sizes",
-                               "left hand side", row_idx.ns_.size(), name,
+  stan::math::check_size_match("matrix[multi,multi] assign rows", name,
+                               row_idx.ns_.size(), "right hand side rows",
                                y_ref.rows());
-  stan::math::check_size_match("matrix[multi,multi] assign column sizes",
-                               "left hand side", col_idx.ns_.size(), name,
+  stan::math::check_size_match("matrix[multi,multi] assign columns", name,
+                               col_idx.ns_.size(), "right hand side columns",
                                y_ref.cols());
   for (int j = 0; j < y_ref.cols(); ++j) {
     const int n = col_idx.ns_[j];
@@ -624,8 +621,8 @@ template <typename Mat1, typename Mat2, typename Idx,
 inline void assign(Mat1&& x, const Mat2& y, const char* name,
                    const Idx& row_idx, const index_multi& col_idx) {
   const auto& y_ref = stan::math::to_ref(y);
-  stan::math::check_size_match("matrix[..., multi] assign column sizes",
-                               "left hand side", col_idx.ns_.size(), name,
+  stan::math::check_size_match("matrix[..., multi] assign column sizes", name,
+                               col_idx.ns_.size(), "right hand side columns",
                                y_ref.cols());
   for (int j = 0; j < col_idx.ns_.size(); ++j) {
     const int n = col_idx.ns_[j];
@@ -684,8 +681,8 @@ inline void assign(Mat1&& x, const Mat2& y, const char* name,
   const auto col_size = x.cols() - start_col;
   stan::math::check_range("matrix[..., min] assign column", name, x.cols(),
                           col_idx.min_);
-  stan::math::check_size_match("matrix[..., min] assign column sizes",
-                               "left hand side", col_size, name, y.cols());
+  stan::math::check_size_match("matrix[..., min] assign columns", name,
+                               col_size, "right hand side columns", y.cols());
   assign(x.rightCols(col_size), y, name, row_idx);
 }
 
@@ -711,11 +708,17 @@ template <typename Mat1, typename Mat2, typename Idx,
           require_dense_dynamic_t<Mat1>* = nullptr>
 inline void assign(Mat1&& x, const Mat2& y, const char* name,
                    const Idx& row_idx, index_max col_idx) {
-  stan::math::check_range("matrix[..., max] assign", name, x.cols(),
-                          col_idx.max_);
-  stan::math::check_size_match("matrix[..., max] assign column size",
-                               "left hand side", col_idx.max_, name, y.cols());
-  assign(x.leftCols(col_idx.max_), y, name, row_idx);
+  if (likely(col_idx.max_ > 0)) {
+    stan::math::check_range("matrix[..., max] assign", name, x.cols(),
+                            col_idx.max_);
+    stan::math::check_size_match("matrix[..., max] assign columns", name,
+                                 col_idx.max_, "right hand side columns",
+                                 y.cols());
+    assign(x.leftCols(col_idx.max_), y, name, row_idx);
+  } else {
+    stan::math::check_size_match("matrix[..., max] assign columns", name, 0,
+                                 "right hand side columns", y.cols());
+  }
 }
 
 /**
@@ -739,23 +742,19 @@ template <typename Mat1, typename Mat2, typename Idx,
           require_dense_dynamic_t<Mat1>* = nullptr>
 inline void assign(Mat1&& x, Mat2&& y, const char* name, const Idx& row_idx,
                    index_min_max col_idx) {
-  stan::math::check_range("matrix[..., min_max] assign min column", name,
-                          x.cols(), col_idx.min_);
-  stan::math::check_range("matrix[..., min_max] assign max column", name,
-                          x.cols(), col_idx.max_);
-  if (col_idx.is_ascending()) {
+  if (likely(col_idx.max_ >= col_idx.min_)) {
+    stan::math::check_range("matrix[..., min_max] assign min column", name,
+                            x.cols(), col_idx.min_);
+    stan::math::check_range("matrix[..., min_max] assign max column", name,
+                            x.cols(), col_idx.max_);
     const auto col_start = col_idx.min_ - 1;
     const auto col_size = col_idx.max_ - col_start;
     stan::math::check_size_match("matrix[..., min_max] assign column size",
-                                 "left hand side", col_size, name, y.cols());
+                                 name, col_size, "right hand side", y.cols());
     assign(x.middleCols(col_start, col_size), y, name, row_idx);
   } else {
-    const auto col_start = col_idx.max_ - 1;
-    const auto col_size = col_idx.min_ - col_start;
-    stan::math::check_size_match("matrix[..., min_max] assign column size",
-                                 "left hand side", col_size, name, y.cols());
-    assign(x.middleCols(col_start, col_size), internal::rowwise_reverse(y),
-           name, row_idx);
+    stan::math::check_size_match("matrix[..., negative_min_max] assign columns",
+                                 name, 0, "right hand side columns", y.cols());
   }
 }
 
@@ -774,8 +773,11 @@ template <typename T, typename U, require_all_std_vector_t<T, U>* = nullptr,
           require_not_t<
               std::is_assignable<std::decay_t<T>&, std::decay_t<U>>>* = nullptr>
 inline void assign(T&& x, U&& y, const char* name) {
-  x.resize(y.size());
-  if (std::is_rvalue_reference<U>::value) {
+  if (unlikely(x.size() != 0)) {
+    stan::math::check_size_match("assign array size", name, x.size(),
+                                 "right hand side", y.size());
+  }
+  if (std::is_rvalue_reference<U&&>::value) {
     for (size_t i = 0; i < y.size(); ++i) {
       assign(x[i], std::move(y[i]), name);
     }
@@ -808,7 +810,7 @@ template <typename StdVec, typename U, typename... Idxs,
           require_std_vector_t<StdVec>* = nullptr>
 inline void assign(StdVec&& x, U&& y, const char* name, index_uni idx1,
                    const Idxs&... idxs) {
-  stan::math::check_range("vector[uni,...] assign", name, x.size(), idx1.n_);
+  stan::math::check_range("array[uni,...] assign", name, x.size(), idx1.n_);
   assign(x[idx1.n_ - 1], std::forward<U>(y), name, idxs...);
 }
 
@@ -831,7 +833,7 @@ inline void assign(StdVec&& x, U&& y, const char* name, index_uni idx1,
 template <typename StdVec, typename U, require_std_vector_t<StdVec>* = nullptr,
           require_t<std::is_assignable<value_type_t<StdVec>&, U>>* = nullptr>
 inline void assign(StdVec&& x, U&& y, const char* name, index_uni idx) {
-  stan::math::check_range("vector[uni,...] assign", name, x.size(), idx.n_);
+  stan::math::check_range("array[uni,...] assign", name, x.size(), idx.n_);
   x[idx.n_ - 1] = std::forward<U>(y);
 }
 
@@ -861,12 +863,26 @@ template <typename T, typename Idx1, typename... Idxs, typename U,
 inline void assign(T&& x, U&& y, const char* name, const Idx1& idx1,
                    const Idxs&... idxs) {
   int x_idx_size = rvalue_index_size(idx1, x.size());
-  stan::math::check_size_match("vector[multi,...] assign", "left hand side",
-                               x_idx_size, name, y.size());
+  // If there is a reverse min_max index or negative max index
+  if (std::is_same<std::decay_t<Idx1>, index_min_max>::value
+      || std::is_same<std::decay_t<Idx1>, index_max>::value) {
+    if (x_idx_size == 0) {
+      if (std::is_same<std::decay_t<Idx1>, index_min_max>::value) {
+        stan::math::check_size_match("array[negative_min_max, ...] assign",
+                                     name, 0, "right hand side", y.size());
+      } else {
+        stan::math::check_size_match("array[max < 1, ...] assign", name, 0,
+                                     "right hand side", y.size());
+      }
+      return;
+    }
+  }
+  stan::math::check_size_match("array[multi, ...] assign", name, x_idx_size,
+                               "right hand side size", y.size());
   for (size_t n = 0; n < y.size(); ++n) {
-    int i = rvalue_at(n, idx1);
-    stan::math::check_range("vector[multi,...] assign", name, x.size(), i);
-    if (std::is_rvalue_reference<U>::value) {
+    size_t i = rvalue_at(n, idx1);
+    stan::math::check_range("array[multi, ...] assign", name, x.size(), i);
+    if (std::is_rvalue_reference<U&&>::value) {
       assign(x[i - 1], std::move(y[n]), name, idxs...);
     } else {
       assign(x[i - 1], y[n], name, idxs...);
