@@ -1,5 +1,11 @@
 #!/usr/bin/env bash
 
+if [ "$#" -ne 1 ]; then
+RUNTESTARGS=""
+else
+RUNTESTARGS="-j"$1
+fi
+
 echo 'Running:'
 echo '  - CmdStan tests'
 echo '  - Stan tests'
@@ -7,14 +13,14 @@ echo '  - Stan Math Library tests'
 echo ''
 echo '------------------------------------------------------------'
 echo 'CmdStan tests'
-./runCmdStanTests.py src/test/interface
+./runCmdStanTests.py $RUNTESTARGS src/test/interface
 
 
 echo ''
 echo '------------------------------------------------------------'
 echo 'Stan tests'
 pushd stan/
-./runTests.py src/test
+./runTests.py $RUNTESTARGS src/test
 popd
 
 
@@ -22,6 +28,6 @@ echo ''
 echo '------------------------------------------------------------'
 echo 'Stan Math Library tests'
 pushd stan/lib/stan_math/
-./runTests.py test/unit
-./runTests.py test/prob
+./runTests.py $RUNTESTARGS test/unit
+./runTests.py $RUNTESTARGS test/prob
 popd
